/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Geoff Beier <me@mollyandgeoff.com>
 *   Based on BrowserTabViewItem.mm by Simon Fraser <sfraser@netscape.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#import <Cocoa/Cocoa.h>


@interface TruncatingTextAndImageCell : NSCell
{
  NSImage         *mImage;
  NSMutableString *mTruncLabelString;
  int             mLabelStringWidth;      // -1 if not known
  float           mImagePadding;
  float           mImageSpace;
  float           mImageAlpha;
  float           mRightGutter;           // leave space for an icon on the right
  float           mMaxImageHeight;
  NSRect          mImageFrame;
  NSView          *mProgressIndicator;
}

-(id)initTextCell:(NSString*)aString;
-(void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView*)controlView;
-(void)setImagePadding:(float)padding;
-(void)setImageSpace:(float)space;
-(void)setImageAlpha:(float)alpha;
-(void)setMaxImageHeight:(float)height;
-(void)setRightGutter:(float)rightPadding;
-(void)setImage:(NSImage *)anImage;
-(NSImage*)image;
-(NSRect)imageFrame;
-(void)addProgressIndicator:(/*NSProgressIndicator * */NSView*)indicator;
-(void)removeProgressIndicator;

@end
