/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla XForms support.
 *
 * The Initial Developer of the Original Code is
 * IBM Corporation.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Allan Beaufour <allan@beaufour.dk>
 *   Alexander Surkov <surkov.alexander@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#include "domstubs.idl"

interface nsIDOMNodeList;
interface nsIXFormsAccessors;

/**
 * Defines additional post-spec. functions for the \<model\> element.
 */
[scriptable, uuid(b1638cdd-db01-4082-a3e9-dc629c469a01)]
interface nsIXFormsNSModelElement : nsISupports
{
  /*
   * Returns a list of the instance documents for the model.
   */
  nsIDOMNodeList getInstanceDocuments();

  /*
   * Returns an accessor interface to the given instance node.
   */
  nsIXFormsAccessors getAccessorsByNode(in nsIDOMNode node);

  /*
   * Evaluates the XPath expression as if it was an XForms single node binding
   * expression for an outer-most control and returns an accessor interface to
   * the resulting node. If XPath expression is invalid then method dispatches
   * "xforms-compute-exception" event to the model and rises an exception. If
   * XPath doesn't point to a node then method rises an exception.
   *
   * @note The expression is only evaluated once.
   */
  nsIXFormsAccessors getAccessorsByExpr(in DOMString xpath);
};
