/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

#ifndef nsGraphicsStatePh_h___
#define nsGraphicsStatePh_h___

#include "nsIRenderingContext.h"
#include "nsIFontMetrics.h"
#include "nsTransform2D.h"
#include "nsRegionPh.h"
#include "nsCOMPtr.h"

class nsGraphicsState
{
public:

  nsTransform2D  *mMatrix;
  nsCOMPtr<nsIRegion> mClipRegion;
  nscolor         mColor;
	nsLineStyle     mLineStyle;
  nsIFontMetrics *mFontMetrics;

  nsGraphicsState *mNext; // link into free list of graphics states.

  friend class nsGraphicsStatePool;

#ifndef USE_GS_POOL
  friend class nsRenderingContextPh;
#endif

private:
  inline nsGraphicsState()
		{
		mMatrix = nsnull;
		mClipRegion = nsnull;
		mColor = NS_RGB(0, 0, 0);
		mLineStyle = nsLineStyle_kSolid;
		mFontMetrics = nsnull;
		}

	inline
  ~nsGraphicsState()
		{
		NS_IF_RELEASE(mFontMetrics);
		}
};

class nsGraphicsStatePool
{
public:

  static
	inline nsGraphicsState * GetNewGS()
		{
  	nsGraphicsStatePool * thePool = PrivateGetPool();
  	return thePool->PrivateGetNewGS();
		}

  static inline void ReleaseGS(nsGraphicsState* aGS)
		{
  	nsGraphicsStatePool * thePool = PrivateGetPool();
  	thePool->PrivateReleaseGS(aGS);
		}
  
  
	inline
  nsGraphicsStatePool() { mFreeList = nsnull; }

  inline ~nsGraphicsStatePool()
		{
  	nsGraphicsState* gs = mFreeList;
  	while (gs != nsnull) {
    	nsGraphicsState* next = gs->mNext;
    	delete gs;
    	gs = next;
  		}
		}
  
private:
  nsGraphicsState*	mFreeList;
  
  inline static nsGraphicsStatePool * PrivateGetPool();
  inline nsGraphicsState *            PrivateGetNewGS();
  inline void                  			  PrivateReleaseGS(nsGraphicsState* aGS);
  
  static nsGraphicsStatePool * gsThePool;
};

#endif /* nsGraphicsStatePh_h___ */
